@@ -22,12 +22,13 @@
 #include <algorithm>
 #include <cassert>
 #include <cstring>
+#include <array>
 
 extern "C" {
-#ifdef LATEST_GREATEST_FFMPEG	
 #include <libavutil/opt.h>
-#endif
 #include <libswscale/swscale.h>
+#include <libavutil/imgutils.h>
+//#include <libavfilter/avfilter.h>
 }
 
 using namespace std;
@@ -35,6 +36,11 @@ using namespace std;
 namespace ffmpegthumbnailer
 {
 
+struct SilenceLogLevel
+{
+    SilenceLogLevel() { av_log_set_level(AV_LOG_QUIET); }
+};
+
 MovieDecoder::MovieDecoder(const string& filename, AVFormatContext* pavContext)
 : m_VideoStream(-1)
 , m_pFormatContext(pavContext)
@@ -63,14 +69,14 @@ void MovieDecoder::initialize(const string& filename)
 
     string inputFile = filename == "-" ? "pipe:" : filename;
     m_AllowSeek = (filename != "-") && (filename.find("rtsp://") != 0) && (filename.find("udp://") != 0);
-    
+
     if ((!m_FormatContextWasGiven) && avformat_open_input(&m_pFormatContext, inputFile.c_str(), nullptr, nullptr) != 0)
     {
         destroy();
         throw logic_error(string("Could not open input file: ") + filename);
     }
 
-	if (avformat_find_stream_info(m_pFormatContext, nullptr) < 0)
+    if (avformat_find_stream_info(m_pFormatContext, nullptr) < 0)
     {
         destroy();
         throw logic_error(string("Could not find stream information"));
@@ -95,7 +101,7 @@ void MovieDecoder::destroy()
 
     if (m_pPacket)
     {
-        av_free_packet(m_pPacket);
+        av_packet_unref(m_pPacket);
         delete m_pPacket;
         m_pPacket = nullptr;
     }
@@ -112,7 +118,7 @@ void MovieDecoder::destroy()
     }
 
     m_VideoStream = -1;
-    
+
     avformat_network_deinit();
 }
 
@@ -140,6 +146,7 @@ void MovieDecoder::initializeVideo()
 
     if (m_VideoStream == -1)
     {
+        destroy();
         throw logic_error("Could not find video stream");
     }
 
@@ -150,17 +157,34 @@ void MovieDecoder::initializeVideo()
     {
         // set to nullptr, otherwise avcodec_close(m_pVideoCodecContext) crashes
         m_pVideoCodecContext = nullptr;
+        destroy();
         throw logic_error("Video Codec not found");
     }
 
     m_pVideoCodecContext->workaround_bugs = 1;
 
-	if (avcodec_open2(m_pVideoCodecContext, m_pVideoCodec, nullptr) < 0)
+    if (avcodec_open2(m_pVideoCodecContext, m_pVideoCodec, nullptr) < 0)
     {
+        destroy();
         throw logic_error("Could not open video codec");
     }
 }
 
+void MovieDecoder::initializeFilterGraph()
+{
+    // avfilter_register_all();
+
+    // auto yadifFilter = avfilter_get_by_name("yadif");
+
+    // AVFilterContext* filterContext;
+    // avfilter_open(&filterContext, yadifFilter, nullptr);
+    // if (avfilter_init_str(filterContext, "\"yadif=1:-1\"") < 0)
+    // {
+    //     destroy();
+    //     throw logic_error("Failed to initialize filter");
+    // }
+}
+
 int MovieDecoder::getWidth()
 {
     if (m_pVideoCodecContext)
@@ -197,7 +221,7 @@ void MovieDecoder::seek(int timeInSeconds)
     {
         return;
     }
-    
+
     int64_t timestamp = AV_TIME_BASE * static_cast<int64_t>(timeInSeconds);
 
     if (timestamp < 0)
@@ -217,7 +241,7 @@ void MovieDecoder::seek(int timeInSeconds)
 
     int keyFrameAttempts = 0;
     bool gotFrame = 0;
-    
+
     do
     {
         int count = 0;
@@ -233,10 +257,10 @@ void MovieDecoder::seek(int timeInSeconds)
             catch(logic_error&) {}
             ++count;
         }
-        
+
         ++keyFrameAttempts;
     } while ((!gotFrame || !m_pFrame->key_frame) && keyFrameAttempts < 200);
-    
+
     if (gotFrame == 0)
     {
         throw logic_error("Seeking in video failed");
@@ -266,10 +290,10 @@ bool MovieDecoder::decodeVideoPacket()
         return false;
     }
 
-    avcodec_get_frame_defaults(m_pFrame);
-    
+    av_frame_unref(m_pFrame);
+
     int frameFinished;
-    
+
     int bytesDecoded = avcodec_decode_video2(m_pVideoCodecContext, m_pFrame, &frameFinished, m_pPacket);
     if (bytesDecoded < 0)
     {
@@ -285,10 +309,10 @@ bool MovieDecoder::getVideoPacket()
     bool frameDecoded = false;
 
     int attempts = 0;
-    
+
     if (m_pPacket)
     {
-        av_free_packet(m_pPacket);
+        av_packet_unref(m_pPacket);
         delete m_pPacket;
     }
 
@@ -302,24 +326,21 @@ bool MovieDecoder::getVideoPacket()
             frameDecoded = m_pPacket->stream_index == m_VideoStream;
             if (!frameDecoded)
             {
-                av_free_packet(m_pPacket);
+                av_packet_unref(m_pPacket);
             }
         }
     }
-    
+
     return frameDecoded;
 }
 
 void MovieDecoder::getScaledVideoFrame(int scaledSize, bool maintainAspectRatio, VideoFrame& videoFrame)
 {
-    if (m_pFrame->interlaced_frame)
-    {
-        avpicture_deinterlace((AVPicture*) m_pFrame, (AVPicture*) m_pFrame, m_pVideoCodecContext->pix_fmt,
-                              m_pVideoCodecContext->width, m_pVideoCodecContext->height);
-    }
+    // TODO: deinterlacing
+
 
     int scaledWidth, scaledHeight;
-    convertAndScaleFrame(PIX_FMT_RGB24, scaledSize, maintainAspectRatio, scaledWidth, scaledHeight);
+    convertAndScaleFrame(AV_PIX_FMT_RGB24, scaledSize, maintainAspectRatio, scaledWidth, scaledHeight);
 
     videoFrame.width = scaledWidth;
     videoFrame.height = scaledHeight;
@@ -330,43 +351,36 @@ void MovieDecoder::getScaledVideoFrame(int scaledSize, bool maintainAspectRatio,
     memcpy((&(videoFrame.frameData.front())), m_pFrame->data[0], videoFrame.lineSize * videoFrame.height);
 }
 
-void MovieDecoder::convertAndScaleFrame(PixelFormat format, int scaledSize, bool maintainAspectRatio, int& scaledWidth, int& scaledHeight)
+void MovieDecoder::convertAndScaleFrame(AVPixelFormat format, int scaledSize, bool maintainAspectRatio, int& scaledWidth, int& scaledHeight)
 {
     calculateDimensions(scaledSize, maintainAspectRatio, scaledWidth, scaledHeight);
 
-#ifdef LATEST_GREATEST_FFMPEG
-	// Enable this when it hits the released ffmpeg version
     SwsContext* scaleContext = sws_alloc_context();
     if (scaleContext == nullptr)
     {
-		throw std::logic_error("Failed to allocate scale context");
-	}
-	
-	av_set_int(scaleContext, "srcw", m_pVideoCodecContext->width);
-    av_set_int(scaleContext, "srch", m_pVideoCodecContext->height);
-    av_set_int(scaleContext, "src_format", m_pVideoCodecContext->pix_fmt);
-    av_set_int(scaleContext, "dstw", scaledWidth);
-    av_set_int(scaleContext, "dsth", scaledHeight);
-    av_set_int(scaleContext, "dst_format", format);
-	av_set_int(scaleContext, "sws_flags", SWS_BICUBIC);
-	
-	const int* coeff = sws_getCoefficients(SWS_CS_DEFAULT);
+        throw std::logic_error("Failed to allocate scale context");
+    }
+
+    av_opt_set_int(scaleContext, "srcw", m_pVideoCodecContext->width, 0);
+    av_opt_set_int(scaleContext, "srch", m_pVideoCodecContext->height, 0);
+    av_opt_set_int(scaleContext, "src_format", m_pVideoCodecContext->pix_fmt, 0);
+    av_opt_set_int(scaleContext, "dstw", scaledWidth, 0);
+    av_opt_set_int(scaleContext, "dsth", scaledHeight, 0);
+    av_opt_set_int(scaleContext, "dst_format", format, 0);
+    av_opt_set_int(scaleContext, "sws_flags", SWS_BICUBIC, 0);
+
+    const int* coeff = sws_getCoefficients(SWS_CS_DEFAULT);
     if (sws_setColorspaceDetails(scaleContext, coeff, m_pVideoCodecContext->pix_fmt, coeff, format, 0, 1<<16, 1<<16) < 0)
     {
-		sws_freeContext(scaleContext);
-		throw std::logic_error("Failed to set colorspace details");
-	}
+        sws_freeContext(scaleContext);
+        throw std::logic_error("Failed to set colorspace details");
+    }
 
-	if (sws_init_context(scaleContext, nullptr, nullptr) < 0)
-	{
-		sws_freeContext(scaleContext);
-		throw std::logic_error("Failed to initialise scale context");
-	}
-#endif
-    
-    SwsContext* scaleContext = sws_getContext(m_pVideoCodecContext->width, m_pVideoCodecContext->height,
-                                              m_pVideoCodecContext->pix_fmt, scaledWidth, scaledHeight,
-                                              format, SWS_BICUBIC, nullptr, nullptr, nullptr);
+    if (sws_init_context(scaleContext, nullptr, nullptr) < 0)
+    {
+        sws_freeContext(scaleContext);
+        throw std::logic_error("Failed to initialise scale context");
+    }
 
     if (nullptr == scaleContext)
     {
@@ -377,26 +391,31 @@ void MovieDecoder::convertAndScaleFrame(PixelFormat format, int scaledSize, bool
     uint8_t* convertedFrameBuffer = nullptr;
 
     createAVFrame(&convertedFrame, &convertedFrameBuffer, scaledWidth, scaledHeight, format);
-    
+
     sws_scale(scaleContext, m_pFrame->data, m_pFrame->linesize, 0, m_pVideoCodecContext->height,
               convertedFrame->data, convertedFrame->linesize);
     sws_freeContext(scaleContext);
 
     av_free(m_pFrame);
     av_free(m_pFrameBuffer);
-    
+
     m_pFrame        = convertedFrame;
     m_pFrameBuffer  = convertedFrameBuffer;
 }
 
 void MovieDecoder::calculateDimensions(int squareSize, bool maintainAspectRatio, int& destWidth, int& destHeight)
 {
+    AVRational par = av_guess_sample_aspect_ratio(m_pFormatContext, m_pVideoStream, m_pFrame);
+
+    // if the pixel aspect ratio is defined and is not 1, we have an anamorphic stream
+    bool anamorphic = par.num != 0 && par.num != par.den;
+
     if (squareSize == 0)
     {
         // use original video size
         squareSize = max(m_pVideoCodecContext->width, m_pVideoCodecContext->height);
     }
-    
+
     if (!maintainAspectRatio)
     {
         destWidth = squareSize;
@@ -406,14 +425,12 @@ void MovieDecoder::calculateDimensions(int squareSize, bool maintainAspectRatio,
     {
         int srcWidth            = m_pVideoCodecContext->width;
         int srcHeight           = m_pVideoCodecContext->height;
-        int ascpectNominator    = m_pVideoCodecContext->sample_aspect_ratio.num;
-        int ascpectDenominator  = m_pVideoCodecContext->sample_aspect_ratio.den;
-        
-        if (ascpectNominator != 0 && ascpectDenominator != 0)
+
+        if (anamorphic)
         {
-            srcWidth = srcWidth * ascpectNominator / ascpectDenominator;
+            srcWidth = srcWidth * par.num / par.den;
         }
-        
+
         if (srcWidth > srcHeight)
         {
             destWidth  = squareSize;
@@ -427,13 +444,14 @@ void MovieDecoder::calculateDimensions(int squareSize, bool maintainAspectRatio,
     }
 }
 
-void MovieDecoder::createAVFrame(AVFrame** pAvFrame, uint8_t** pFrameBuffer, int width, int height, PixelFormat format)
+void MovieDecoder::createAVFrame(AVFrame** pAvFrame, uint8_t** pFrameBuffer, int width, int height, AVPixelFormat format)
 {
     *pAvFrame = av_frame_alloc();
+    auto* avPicture = reinterpret_cast<AVPicture*>(*pAvFrame);
 
-    int numBytes = avpicture_get_size(format, width, height);
+    int numBytes = av_image_get_buffer_size(format, width, height, 1);
     *pFrameBuffer = reinterpret_cast<uint8_t*>(av_malloc(numBytes));
-    avpicture_fill((AVPicture*) *pAvFrame, *pFrameBuffer, format, width, height);
+    av_image_fill_arrays(avPicture->data, avPicture->linesize, *pFrameBuffer, format, width, height, 1);
 }
 
 }
