@@ -14,9 +14,7 @@
 //    along with this program; if not, write to the Free Software
 //    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "videothumbnailer.h"
 
@@ -58,6 +56,7 @@ VideoThumbnailer::VideoThumbnailer()
 VideoThumbnailer::VideoThumbnailer(int thumbnailSize, bool workaroundIssues, bool maintainAspectRatio, int imageQuality, bool smartFrameSelection)
 : m_ThumbnailSize(thumbnailSize)
 , m_SeekPercentage(10)
+, m_OverlayFilmStrip(false)
 , m_WorkAroundIssues(workaroundIssues)
 , m_ImageQuality(imageQuality)
 , m_MaintainAspectRatio(maintainAspectRatio)
@@ -117,7 +116,7 @@ void VideoThumbnailer::generateThumbnail(const string& videoFile, ImageWriter& i
 {
     MovieDecoder movieDecoder(videoFile, pAvContext);
     movieDecoder.decodeVideoFrame(); //before seeking, a frame has to be decoded
- 
+
     if ((!m_WorkAroundIssues) || (movieDecoder.getCodec() != "h264")) //workaround for bug in older ffmpeg (100% cpu usage when seeking in h264 files)
     {
         try
@@ -126,21 +125,31 @@ void VideoThumbnailer::generateThumbnail(const string& videoFile, ImageWriter& i
 													: timeToSeconds(m_SeekTime);
             movieDecoder.seek(secondToSeekTo);
         }
-        catch (exception& e)
+        catch (const exception& e)
         {
-            cerr << e.what() << ", will use first frame" << endl;
+            TraceMessage(ThumbnailerLogLevelError, std::string(e.what()) + ", will use first frame.");  
+            
             //seeking failed, try the first frame again
             movieDecoder.destroy();
             movieDecoder.initialize(videoFile);
             movieDecoder.decodeVideoFrame();
          }
     }
-    
+
     VideoFrame  videoFrame;
-    
+
     if (m_SmartFrameSelection)
     {
-        generateSmartThumbnail(movieDecoder, videoFrame);
+        try
+        {
+            generateSmartThumbnail(movieDecoder, videoFrame);
+        }
+        catch (const exception& e)
+        {
+            TraceMessage(ThumbnailerLogLevelError, std::string(e.what()) + ". Smart frame selection failed. Retrying without smart frame detection.");  
+            m_SmartFrameSelection = false;
+            generateThumbnail(videoFile, imageWriter, pAvContext);
+        }
     }
     else
     {
@@ -162,16 +171,16 @@ void VideoThumbnailer::generateSmartThumbnail(MovieDecoder& movieDecoder, VideoF
 {
     vector<VideoFrame> videoFrames(SMART_FRAME_ATTEMPTS);
     vector<Histogram<int> > histograms(SMART_FRAME_ATTEMPTS);
-    
+
     for (int i = 0; i < SMART_FRAME_ATTEMPTS; ++i)
     {
         movieDecoder.decodeVideoFrame();
         movieDecoder.getScaledVideoFrame(m_ThumbnailSize, m_MaintainAspectRatio, videoFrames[i]);
         generateHistogram(videoFrames[i], histograms[i]);
     }
-    
+
     int bestFrame = getBestThumbnailIndex(videoFrames, histograms);
-    
+
     assert(bestFrame != -1);
     videoFrame = videoFrames[bestFrame];
 }
@@ -193,7 +202,10 @@ void VideoThumbnailer::generateThumbnail(const string& videoFile, ThumbnailerIma
 
 void VideoThumbnailer::writeImage(const string& videoFile, ImageWriter& imageWriter, const VideoFrame& videoFrame, int duration, vector<uint8_t*>& rowPointers)
 {
-    if((videoFile != "-") && (videoFile.find("rtsp://") != 0) && (videoFile.find("udp://") != 0) && (videoFile.find("http://") != 0))
+    if ((videoFile != "-") &&
+        (videoFile.compare(0, 7, "rtsp://") != 0) &&
+        (videoFile.compare(0, 6, "udp://") != 0) &&
+        (videoFile.compare(0, 7, "http://") != 0))
     {
         struct stat statInfo;
         if (stat(videoFile.c_str(), &statInfo) == 0)
@@ -205,7 +217,7 @@ void VideoThumbnailer::writeImage(const string& videoFile, ImageWriter& imageWri
         {
             cout << "Warn: Failed to stat file " << videoFile << " (" << strerror(errno) << ")" << endl;
         }
-        
+
         string mimeType = getMimeType(videoFile);
         if (!mimeType.empty())
         {
@@ -215,6 +227,7 @@ void VideoThumbnailer::writeImage(const string& videoFile, ImageWriter& imageWri
         imageWriter.setText("Thumb::URI", videoFile);
         imageWriter.setText("Thumb::Movie::Length", StringOperations::toString(duration));
     }
+
     imageWriter.writeFrame(&(rowPointers.front()), videoFrame.width, videoFrame.height, m_ImageQuality);
 }
 
@@ -242,7 +255,7 @@ string VideoThumbnailer::getMimeType(const string& videoFile)
     {
         return "video/x-ms-wm";
     }
-    else if (extension == "mp4")
+    else if (extension == "wmv")
     {
         return "video/x-ms-wmv";
     }
@@ -250,6 +263,10 @@ string VideoThumbnailer::getMimeType(const string& videoFile)
     {
         return "video/mp4";
     }
+    else if (extension == "webm")
+    {
+        return "video/webm";
+    }
     else if (extension == "flv")
     {
         return "video/x-flv";
@@ -263,7 +280,7 @@ string VideoThumbnailer::getMimeType(const string& videoFile)
 string VideoThumbnailer::getExtension(const string& videoFilename)
 {
     string extension;
-    string::size_type pos = videoFilename.rfind('.');
+    auto pos = videoFilename.rfind('.');
 
     if (string::npos != pos)
     {
@@ -280,14 +297,9 @@ void VideoThumbnailer::addFilter(IFilter* pFilter)
 
 void VideoThumbnailer::removeFilter(IFilter* pFilter)
 {
-    for (vector<IFilter*>::iterator iter = m_Filters.begin(); iter != m_Filters.end(); ++iter)
-    {
-        if (*iter == pFilter)
-        {
-            m_Filters.erase(iter);
-            break;
-        }
-    }
+    std::remove_if(m_Filters.begin(), m_Filters.end(), [pFilter] (IFilter* fil) {
+        return fil == pFilter;
+    });
 }
 
 void VideoThumbnailer::clearFilters()
@@ -295,11 +307,16 @@ void VideoThumbnailer::clearFilters()
     m_Filters.clear();
 }
 
+void VideoThumbnailer::setLogCallback(std::function<void(ThumbnailerLogLevel, const std::string&)> cb)
+{
+    m_LogCb = cb;
+}
+
 void VideoThumbnailer::applyFilters(VideoFrame& frameData)
 {
-    for (vector<IFilter*>::iterator iter = m_Filters.begin(); iter != m_Filters.end(); ++iter)
+    for (auto filter : m_Filters)
     {
-        (*iter)->process(frameData);
+        filter->process(frameData);
     }
 }
 
@@ -329,7 +346,7 @@ int VideoThumbnailer::getBestThumbnailIndex(vector<VideoFrame>& videoFrames, con
             avgHistogram.b[j] += static_cast<float>(histograms[i].b[j]) / histograms.size();
         }
     }
-    
+
     int bestFrame = -1;
     float minRMSE = FLT_MAX;
     for (size_t i = 0; i < histograms.size(); ++i)
@@ -343,14 +360,14 @@ int VideoThumbnailer::getBestThumbnailIndex(vector<VideoFrame>& videoFrames, con
                         + fabsf(avgHistogram.b[j] - histograms[i].b[j]);
             rmse += (error * error) / 255;
         }
-        
+
         rmse = sqrtf(rmse);
         if (rmse < minRMSE)
         {
             minRMSE = rmse;
             bestFrame = i;
         }
-        
+
 #ifdef DEBUG_MODE
         stringstream outputFile;
         outputFile << "frames/Frame" << setfill('0') << setw(3) << i << "_" << rmse << endl;
@@ -362,7 +379,7 @@ int VideoThumbnailer::getBestThumbnailIndex(vector<VideoFrame>& videoFrames, con
         delete imageWriter;
 #endif
     }
-    
+
 #ifdef DEBUG_MODE
     cout << "Best frame was: " << bestFrame << "(RMSE: " << minRMSE << ")" << endl;
 #endif
@@ -370,5 +387,13 @@ int VideoThumbnailer::getBestThumbnailIndex(vector<VideoFrame>& videoFrames, con
     return bestFrame;
 }
 
+void VideoThumbnailer::TraceMessage(ThumbnailerLogLevel lvl, const std::string& msg)
+{
+    if (m_LogCb)
+    {
+        m_LogCb(lvl, msg);
+    }
+}
+
 }
 
